// "How to do it" test for window without a title bar

#include NoTitleBar.h

#DEFINE TITLEBARHEIGHT	40

DATA SECTION
	// Instance handle for this process
	hInstance		HANDLE	?
	// Handle of the main dialog
	hDlg			HANDLE	?
	// Flag to indicate the the window is being dragged
	IsDragging		BOOL	?
	// Private class name of main window
	ClassName		CHAR	"NoTitleClass",0
	// Holder for POINT that indicates how far the cursor is
	// from the top left corner of the window
	OffsetPoint		POINT	<?>

	hRegion			HANDLE	?

CODE SECTION

Start:
	// Get the instance handle
	invoke GetModuleHandle,NULL
	mov [hInstance],eax
	// Call the window creation and message pump procedure
	invoke WinMain,[hInstance],NULL,NULL,TRUE
	// Terminate
	invoke ExitProcess,0

WinMain FRAME hInst,hPrevInstance,lpCmdLine,nCmdShow
	LOCAL	wc				:WNDCLASSEX
	LOCAL	msg				:MSG
	// Handle of the close button bitmap
	LOCAL	hCloseBmp		:%HANDLE
	LOCAL	rct				:RECT

	// Define our main window class and register it
	mov		D[wc.cbSize], SIZEOF WNDCLASSEX
	mov		D[wc.style], CS_HREDRAW + CS_VREDRAW
	mov		D[wc.lpfnWndProc], OFFSET DlgProc
	mov		D[wc.cbClsExtra], NULL
	mov		D[wc.cbWndExtra], DLGWINDOWEXTRA
	mov		eax, [hInstance]
	mov		[wc.hInstance], eax
	// Note that a NULL brush is used, the window is painted
	// using WM_PAINT, it has to be transparent so the rounded
	// corners will work.
	mov		D[wc.hbrBackground], NULL
	mov		D[wc.lpszMenuName], NULL
	mov		D[wc.lpszClassName], OFFSET ClassName
	invoke	LoadIcon, NULL, IDI_APPLICATION
	mov		[wc.hIcon], eax
	mov		[wc.hIconSm], eax
	invoke	LoadCursor,NULL,IDC_ARROW
	mov		[wc.hCursor], eax

	invoke	RegisterClassEx, offset wc
	or eax,eax
	jnz >
		// If unable to register the class shut down
		invoke	ExitProcess, -1
	:

	invoke	CreateDialogParam, [hInstance],1000, NULL, NULL, NULL
	or eax,eax
	jnz >
		// If unable to create the dialog shut down
		invoke	ExitProcess, -1
	:
	mov [hDlg],eax

	// Load the image for the close button
	invoke LoadImage,[hInstance],100,IMAGE_BITMAP,0,0,LR_DEFAULTCOLOR
	mov [hCloseBmp],eax
	// Assign the image to the button
	invoke SendDlgItemMessage,[hDlg],1001,BM_SETIMAGE,IMAGE_BITMAP,[hCloseBmp]

	invoke ShowWindow,[hDlg],[nCmdShow]


	// Get the size of the window. Since there's no non-client area
	// the total size is equal to the client area
	invoke GetClientRect,[hDlg],offset rct

	// Create a region that matches the client area but with
	// rounded corners

	// Kludge for XP
	mov eax,[rct.right]
	inc eax
	mov ecx,[rct.bottom]
	inc ecx

	invoke CreateRoundRectRgn,[rct.left],[rct.top],eax,ecx,10,10
	mov [hRegion],eax
	
	invoke SetWindowRgn,[hDlg],[hRegion],TRUE


	// The message pump
	:
		invoke GetMessage, offset msg,NULL,NULL,NULL
		or eax,eax
		jz >>
			invoke IsDialogMessage, [hDlg], offset msg
			or eax,eax
			jnz <<
				invoke TranslateMessage, offset msg
				invoke DispatchMessage, offset msg
				jmp <<
	:

	// Delete the close bitmap
	invoke DeleteObject,[hCloseBmp]
	invoke DeleteObject,[hRegion]

	mov eax,[msg.wParam]

	ret
ENDF

DlgProc FRAME hwnd,uMsg,wParam,lParam
	LOCAL ptc:POINT
	LOCAL rct:RECT
	LOCAL ps:PAINTSTRUCT
	LOCAL hrgn:%HANDLE
	LOCAL hmemdc:%HANDLE
	LOCAL hbmp:%HANDLE
	LOCAL hbrsh:%HANDLE
	
	LOCAL hDskDC:%HANDLE
	LOCAL WndRect:RECT

	.WM_MOUSEMOVE
	cmp D[uMsg],WM_MOUSEMOVE
	jne >>.WM_PAINT
		// If the window is dragging then move it with the mouse
		cmp D[IsDragging],TRUE
		jne >>.DEFPROC
		// Get the current cusor position and adjust it to
		// the top left corner of the window
		invoke GetCursorPos,offset ptc
		mov eax,[OffsetPoint.x]
		sub [ptc.x],eax
		mov eax,[OffsetPoint.y]
		sub [ptc.y],eax
		// Move the window to its new position
		invoke SetWindowPos,[hwnd],0,[ptc.x],[ptc.y],0,0,SWP_NOSIZE + SWP_NOZORDER

		xor eax,eax
		ret

	.WM_PAINT
	cmp D[uMsg],WM_PAINT
	jne >>.WM_LBUTTONDOWN
		// The window is painted here, call BeginPaint to start
		invoke BeginPaint,[hwnd],offset ps

			// Make it pretty

			// Get the size of the window. Since there's no non-client area
			// the total size is equal to the client area
			invoke GetClientRect,[hwnd],offset rct

			// A memory DC is used for our drawing
			invoke CreateCompatibleDC,[ps.hdc]
			mov [hmemdc],eax
			// Create a bitmap to draw on
			invoke CreateCompatibleBitmap,[ps.hdc],[rct.right],[rct.bottom]
			mov [hbmp],eax
			// Select the bitmap into the memory DC and push the old bitmap object handle
			invoke SelectObject,[hmemdc],eax
			push eax

			// Call the fill procedure to paint the bitmap
			invoke GradientFillRect,[hmemdc],0FFFFF8F0h,0FFC0C0C0h,0,0,[rct.right],[rct.bottom],1
			push [rct.right]
			push [rct.bottom]
			// Draw a thin edge to indicate the new title bar
			// The window will only move if dragged from within
			// the new title area - See WM_LBUTTONDOWN for details
			mov D[rct.left],0
			mov D[rct.top],TITLEBARHEIGHT
			mov D[rct.bottom],TITLEBARHEIGHT
			invoke DrawEdge,[hmemdc],OFFSET rct,EDGE_ETCHED,BF_TOP

			invoke LoadIcon, NULL, IDI_APPLICATION
			invoke DrawIconEx,[hmemdc],10,5,eax,(TITLEBARHEIGHT-10),(TITLEBARHEIGHT-10),NULL,NULL,DI_NORMAL

			pop [rct.bottom]
			pop [rct.right]

			// Blt the backbuffer to the window
			invoke BitBlt,[ps.hdc],0,0,[rct.right],[rct.bottom],[hmemdc],0,0,SRCCOPY

			// pop the old bitmap object handle
			pop eax
			// Select it into the DC, this will release hbmp and return its handle
			invoke SelectObject,[hmemdc],eax

			// Clean up the objects we used in the paint process
			invoke DeleteObject,[hbmp]
			invoke DeleteDC,[hmemdc]
		// Shut down the painting and return FALSE
		invoke EndPaint,[hwnd],offset ps
		xor eax,eax
		ret

	.WM_LBUTTONDOWN
	cmp D[uMsg],WM_LBUTTONDOWN
	jne >>.WM_LBUTTONUP

		// The left button is down check to see if its
		// in the new title area
		invoke GetCursorPos,offset ptc
		invoke ScreenToClient,[hwnd],offset ptc
		cmp D[ptc.y],TITLEBARHEIGHT
		jg >>.DEFPROC

		invoke SetCapture,[hwnd]
		// The left button has been pressed in the new title 
		// area of the main window.
		// Calculate the offset from the top-left corner and
		// save the result in OffsetPoint. Use screen coordinates
		// for the calculation.

		invoke GetCursorPos,offset OffsetPoint
		
		invoke GetWindowRect,[hwnd],offset rct
		mov eax,[OffsetPoint.x]
		sub eax,[rct.left]
		mov [OffsetPoint.x],eax
		mov eax,[OffsetPoint.y]
		sub eax,[rct.top]
		mov [OffsetPoint.y],eax

		// Set the dragging flag to TRUE, any mouse movement
		// will cause the window to move as well until this
		// flag is reset when the mouse button is released
		mov D[IsDragging],TRUE
		xor eax,eax
		ret

	.WM_LBUTTONUP
	cmp D[uMsg],WM_LBUTTONUP
	jne >>.WM_COMMAND
		// Reset the dragging flag
		cmp D[IsDragging],FALSE
		je >>.DEFPROC
		invoke SetCapture,NULL
	;	invoke InvalidateRect,[hwnd],NULL,TRUE
		mov D[IsDragging],FALSE
		xor eax,eax
		ret

	.WM_COMMAND
	cmp D[uMsg],WM_COMMAND
	jne >>.WM_DESTROY
		mov eax,[wParam]
		cmp eax,1001
		jne >>.DEFPROC
			invoke PostMessage,[hwnd],WM_CLOSE,0,0
			xor eax,eax
			ret

	.WM_DESTROY
	cmp S[uMsg],WM_DESTROY
	jne >.DEFPROC
		invoke PostQuitMessage,0
		// Let this message fall through to the default
		// window proc for any additional processing

	.DEFPROC
		invoke DefWindowProc,[hwnd],[uMsg],[wParam],[lParam]
		ret

ENDF

GradientFillRect FRAME hDC,Color1,Color2,x,y,w,h,Direction
	LOCAL vert[2]:TRIVERTEX
	LOCAL gRect:GRADIENT_RECT
	LOCAL gFillMethod:D

	mov eax,[x]
	mov [vert.x],eax
	mov eax,[y]
	mov [vert.y],eax

	mov eax,[Color1]

	xor ecx,ecx
	movzx ecx,al
	shl ecx,8
	mov [vert.Red],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Green],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Blue],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Alpha],cx

	mov eax,[w]
	add eax,[x]
	mov [vert.x + SIZEOF TRIVERTEX],eax
	mov eax,[h]
	add eax,[y]
	mov [vert.y + SIZEOF TRIVERTEX],eax

	mov eax,[Color2]

	xor ecx,ecx
	movzx ecx,al
	shl ecx,8
	mov [vert.Red + SIZEOF TRIVERTEX],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Green + SIZEOF TRIVERTEX],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Blue + SIZEOF TRIVERTEX],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Alpha + SIZEOF TRIVERTEX],cx

	mov D[gRect.UpperLeft],0
	mov D[gRect.LowerRight],1

	cmp D[Direction],FALSE
	jne >
		mov D[gFillMethod],GRADIENT_FILL_RECT_H
		jmp >.FILL
	:
		mov D[gFillMethod],GRADIENT_FILL_RECT_V
	.FILL

	invoke GradientFill,[hDC],ADDR vert,2,ADDR gRect,1,[gFillMethod]

	ret

ENDF

